"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const changeCase = require('change-case');
const schema_1 = require('./schema');
let database = null;
let connected = false;
exports.schema = new schema_1.Schema();
function snakeFieldName(str) {
    return str.split('.').map(name => changeCase.snake(name)).join('.');
}
function setDatabase(db) {
    database = db;
    database.schema = exports.schema;
}
exports.setDatabase = setDatabase;
function connect() {
    return __awaiter(this, void 0, void 0, function* () {
        if (connected)
            return;
        yield database.connect();
        connected = true;
    });
}
exports.connect = connect;
function build() {
    return __awaiter(this, void 0, void 0, function* () {
        yield connect();
        yield database.createSchema();
    });
}
exports.build = build;
function dumpSchema() {
    return __awaiter(this, void 0, void 0, function* () {
        yield connect();
        exports.schema.dump();
    });
}
exports.dumpSchema = dumpSchema;
class Repository {
    constructor(entity) {
        this.entity = entity;
        this.table = exports.schema.getTable(entity);
    }
    query() {
        return new QueryBuilder(this.entity);
    }
    findOne(extra) {
        return __awaiter(this, void 0, void 0, function* () { return this.query().findOne(extra); });
    }
    findAll(extra) {
        return __awaiter(this, void 0, void 0, function* () { return this.query().findAll(extra); });
    }
    insert(item) {
        return __awaiter(this, void 0, void 0, function* () { return this.query().insert(item); });
    }
    delete(item) {
        return __awaiter(this, void 0, void 0, function* () { return this.query().delete(item); });
    }
}
exports.Repository = Repository;
class QueryBuilder {
    constructor(entity) {
        this.entity = entity;
        this.query = {
            target: entity
        };
        this.table = exports.schema.getTable(entity);
    }
    /* finding results */
    findOne(extra) {
        return __awaiter(this, void 0, void 0, function* () {
            // this.query.limit = 1;
            // issue: limit = 1 wouldn't work with X-to-many assoc joins
            let arr = extra && [];
            let result = yield database.findAll(this.query, arr);
            if (result.length > 0) {
                for (let key in arr[0]) {
                    extra[key] = arr[0][key];
                }
                return result[0];
            }
            else {
                return null;
            }
        });
    }
    findAll(extra) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield database.findAll(this.query, extra);
        });
    }
    insert(item) {
        return __awaiter(this, void 0, void 0, function* () {
            yield database.insert(this.query, item);
        });
    }
    delete(item) {
        return __awaiter(this, void 0, void 0, function* () {
            yield database.delete(this.query, item);
        });
    }
    total() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield database.total(this.query);
        });
    }
    /* query params */
    distinct() {
        this.query.distinct = true;
        return this;
    }
    limit(limit) {
        this.query.limit = limit;
        return this;
    }
    offset(offset) {
        this.query.offset = offset;
        return this;
    }
    sortBy(a, b, c) {
        let entity = this.query.target;
        let field;
        let order = 'asc';
        if (c != null) {
            entity = a;
            field = b;
            order = c;
        }
        else if (typeof b == 'string') {
            field = a;
            order = b;
        }
        else if (b != null) {
            entity = a;
            field = b;
        }
        else {
            field = a;
        }
        let table = exports.schema.getTable(entity);
        if (!this.query.orders)
            this.query.orders = [];
        this.query.orders.push({
            field: `${this.table.name}.${snakeFieldName(field)}`,
            order: order
        });
        return this;
    }
    groupBy(a, b) {
        let entity = this.query.target;
        let field;
        if (b != null) {
            entity = a;
            field = b;
        }
        else if (typeof a == 'object') {
            this.query.groups.push(a);
            return this;
        }
        else {
            field = a;
        }
        let table = exports.schema.getTable(entity);
        if (!this.query.groups)
            this.query.groups = [];
        let node = {
            type: 'field',
            field: `${this.table.name}.${field}`
        };
        this.query.groups.push(node);
        return this;
    }
    alias(node, alias) {
        if (!this.query.aliases)
            this.query.aliases = [];
        this.query.aliases.push({ node, alias });
    }
    /* conditions */
    where(...expr) {
        return this.andWhere(...expr);
    }
    andWhere(...expr) {
        for (let item of expr) {
            if (!this.query.condition) {
                this.query.condition = item;
            }
            else {
                this.query.condition = this.and(this.query.condition, item);
            }
        }
        return this;
    }
    orWhere(...expr) {
        for (let item of expr) {
            if (!this.query.condition) {
                this.query.condition = item;
            }
            else {
                this.query.condition = this.or(this.query.condition, item);
            }
        }
        return this;
    }
    and(...expr) {
        return this.commonLogical('and', ...expr);
    }
    or(...expr) {
        return this.commonLogical('or', ...expr);
    }
    commonLogical(type, ...expr) {
        let root = null;
        for (let item of expr) {
            if (!root) {
                root = item;
            }
            else {
                let link = {
                    type: type,
                    left: root,
                    right: item
                };
                root = link;
            }
        }
        return root;
    }
    eq(left, right) {
        return this.commonBinary('eq', left, right);
    }
    neq(left, right) {
        return this.commonBinary('neq', left, right);
    }
    lt(left, right) {
        return this.commonBinary('lt', left, right);
    }
    le(left, right) {
        return this.commonBinary('le', left, right);
    }
    gt(left, right) {
        return this.commonBinary('gt', left, right);
    }
    ge(left, right) {
        return this.commonBinary('ge', left, right);
    }
    commonBinary(type, left, right) {
        if (typeof left == 'string') {
            left = this.field(left);
        }
        if (typeof right == 'string') {
            left = this.field(right);
        }
        return {
            type: type,
            left: left,
            right: right
        };
    }
    /* functions */
    sum(param) {
        if (typeof param == 'string') {
            return {
                type: 'call',
                function: 'sum',
                parameters: [this.field(param)]
            };
        }
        else {
            return {
                type: 'call',
                function: 'sum',
                parameters: [param]
            };
        }
    }
    quote(value) {
        return {
            type: 'literal',
            value: value
        };
    }
    field(name) {
        return {
            type: 'field',
            field: `${this.table.name}.${snakeFieldName(name)}`
        };
    }
}
exports.QueryBuilder = QueryBuilder;
//# sourceMappingURL=orm.js.map