"use strict";
const changeCase = require('change-case');
class Schema {
    constructor() {
        this.tables = new Map();
    }
    getTable(entity) {
        let table;
        if (!this.tables.has(entity)) {
            table = new Table(entity);
            this.tables.set(entity, table);
        }
        else {
            table = this.tables.get(entity);
        }
        return table;
    }
    convertItemToObject(entity) {
        let table = this.getTable(entity.constructor);
        let data = {};
        for (let field of table.fields.values()) {
            data[field.internalName] = entity[field.name];
        }
        return data;
    }
    convertObjectToItem(data, entity, prefix, item) {
        let table = this.getTable(entity);
        if (table.idField) {
            if (data[`${prefix}.${table.idField.internalName}`] == null) {
                return null;
            }
        }
        if (!item)
            item = new entity();
        if (table.extends) {
            this.convertObjectToItem(data, table.extends.entity, `${prefix}._base`, item);
        }
        for (let field of table.fields.values()) {
            if (field.associatedField) {
                let refEntity = field.associatedField.table.entity;
                let refPrefix = `${prefix}.${field.internalName}`;
                let refItem = this.convertObjectToItem(data, refEntity, refPrefix);
                if (refItem) {
                    if (field.associatedField.isArray) {
                        if (!refItem[field.associatedField.name])
                            refItem[field.associatedField.name] = [];
                        if (item)
                            refItem[field.associatedField.name].push(item);
                    }
                    else {
                        refItem[field.associatedField.name] = item;
                    }
                }
                if (field.isArray) {
                    if (!item[field.name])
                        item[field.name] = [];
                    if (refItem)
                        item[field.name].push(refItem);
                }
                else {
                    item[field.name] = refItem;
                }
            }
            else {
                item[field.name] = data[`${prefix}.${field.internalName}`];
            }
        }
        return item;
    }
    dump() {
        let stream = process.stdout;
        stream.write(`Schema dump\n\n`);
        for (let table of this.tables.values()) {
            stream.write(`Table: ${table.name}`);
            if (table.extends)
                stream.write(` extends ${table.extends.name}`);
            stream.write(`\n`);
            for (let field of table.fields.values()) {
                let a = field == table.idField ? '*' : '';
                if (field.associatedField) {
                    stream.write(`\tField: ${a}${field.internalName} -> `);
                    if (field.leading)
                        stream.write(`*`);
                    stream.write(`${field.associatedField.table.name}.`);
                    stream.write(`${field.associatedField.internalName}`);
                    if (field.isArray)
                        stream.write(`[]`);
                    stream.write(`\n`);
                }
                else {
                    stream.write(`\tField: ${a}${field.internalName}\n`);
                }
            }
            stream.write(`\n`);
        }
    }
}
exports.Schema = Schema;
/**
 * @brief Abstract database table.
 */
class Table {
    constructor(entity) {
        this.entity = entity;
        this.fields = new Map();
        this.name = changeCase.snake(entity.name);
        this.idField = null;
        this.idFields = [];
        this.extends = null;
        this.extendedBy = [];
    }
    createField(name, type) {
        let field = new Field(this, name, type);
        this.fields.set(field.internalName, field);
        return field;
    }
    getField(name) {
        name = changeCase.snake(name);
        return this.fields.get(name);
    }
    hasField(name) {
        name = changeCase.snake(name);
        return this.fields.has(name);
    }
}
exports.Table = Table;
/**
 * @brief Abstract database table field.
 */
class Field {
    constructor(table, name, type) {
        this.table = table;
        this.name = name;
        this.type = type;
        this.internalName = changeCase.snake(name);
        this.associatedField = null;
        this.isArray = false;
        this.joinTable = null;
    }
    associate(field) {
        this.associatedField = field;
        this.leading = true;
        field.associatedField = this;
        field.leading = false;
        if (this.isArray && field.isArray) {
            let tableName;
            if (this.table.name < field.table.name) {
                tableName = `${this.table.name}_${this.internalName}`;
            }
            else {
                tableName = `${field.table.name}_${field.internalName}`;
            }
            this.joinTable = field.joinTable = tableName;
        }
    }
}
exports.Field = Field;
//# sourceMappingURL=schema.js.map